#!/bin/bash

me=$(readlink -f $0)
BASE=$(dirname ${me})

if test "$(lsb_release -is)" != "CentOS";then
    read -p "Wrong OS! Please download the Ubuntu or combined update from http://www.ferrari-electronic.de. [Abort]" answer
    exit 2
fi

featureversion=$(featureversion)

if [ ! -z "${featureversion}" ]; then
    if [ -e /tmp/gateconfig.featureversion ]; then
        gateconfigversion=$(cat /tmp/gateconfig.featureversion)
        if [ -z "${gateconfigversion}" -o ${gateconfigversion} -lt ${featureversion} ]; then
            echo "***********************************************************************************"
            echo "* OfficeMaster GateConfiguration too old. Please download new version"
            echo "* http://www.ferrari-electronic.de/fileadmin/download_center/OfficeMasterHardware/Setup-OfficeMaster-Gate-Config.exe"
            echo "***********************************************************************************"
            read -p "OfficeMaster GateConfiguration too old. Please download new version. [Abort/Default=Abort]!" answer
            sleep 5
            echo "## Installation failed ## ..."
            echo "FLASH_FAILED" # Old gateConfig
            touch /tmp/flashfailed
            exit 1
        fi
    else
        if [ -e /var/config.active -o "${HISTFILE}" != "" ]; then
            echo "***********************************************************************************"
            echo "* OfficeMaster GateConfiguration too old. Please download new version"
            echo "* http://www.ferrari-electronic.de/fileadmin/download_center/OfficeMasterHardware/Setup-OfficeMaster-Gate-Config.exe"
            echo "***********************************************************************************"
            read -p "OfficeMaster GateConfiguration too old. Please download new version. [Abort/Default=Abort]!" answer
            sleep 5
            echo "## Installation failed ## ..."
            echo "FLASH_FAILED" # Old gateConfig
            touch /tmp/flashfailed
            exit 2
        fi
    fi
fi

function flashit {
    OSType=$(bash -c "if [ -r /etc/centos-release ] ; then cat /etc/centos-release | head -n 1 | cut -d ' ' -f 1 | sed 's/.*/\L&/' ; elif [ -r /etc/issue ] ; then cat /etc/issue | head -n 1 | cut -d ' ' -f 1 | sed 's/.*/\L&/'; else echo unknown ; fi")

    if [ $OSType = "centos" ]; then
	echo "CentOS detected"
    elif [ $OSType = "red" ]; then
	echo "RedHat detected"
    elif [ $OSType = "ubuntu" ]; then
	echo "Ubuntu detected"
	echo "Check software..."
	if ! dpkg -s omg-userfs >/dev/null 2>&1; then
	    echo "Error: userfs missing. Perform omgupdate first."
	    return 2
	fi
	if ! dpkg -s omg-modules >/dev/null 2>&1; then
	    echo "Error: modules missing. Perform omgupdate first."
	    return 2
	fi
	if ! dpkg -s omg-utilities >/dev/null 2>&1; then
	    echo "Error: utilities missing. Perform omgupdate first."
	    return 2
	fi
    else
	echo "unknown OS"
	return 3
    fi

    if ! which omgstopall > /dev/null; then
	echo "Error: omgstopall misssing. Perform omgupdate first."
	return 2
    fi
    if ! which wdstop > /dev/null; then
	echo "Error: wdstop missing. Perform omgupdate first."
	return 2
    fi
    if ! which cfgFlash > /dev/null; then
	echo "Error: cfgFlash  misssing. Perform omgupdate first."
	return 2
    fi

    echo "Stopping software..."
    omgstop 2>&1 >/dev/null
    wdstop 2>/dev/null

    modprobe i2c-dev 2>/dev/null
    modprobe tdm_dma 2>/dev/null
    modprobe tdm_i2c 2>/dev/null

    for i in $(getchannels -a 2>&1| grep '^0\.[a-z]/X/CFG');do
	hw=$(echo $i | cut -d '.' -f 2 | cut -d '/' -f 1)
	case "${hw}" in
	    "a")
		device="/dev/tdmSpi8" 
		;;
	    "b")
		device="/dev/tdmSpi108" 
		;;
	    "c")
		device="/dev/tdmSpi208" 
		;;
	    "d")
		device="/dev/tdmSpi308" 
		;;
	    "e")
		device="/dev/tdmSpi408" 
		;;
	    "f")
		device="/dev/tdmSpi508" 
		;;
	    "g")
		device="/dev/tdmSpi608" 
		;;
	    "h")
		device="/dev/tdmSpi708" 
		;;
	    *)
		echo "cant calculate apropiate device!"
		return 3
		;;
	esac
	
	let version=0x$(echo fpga_version | confTool /dev/ilid ${i}| head -1| cut -f 3 -d ' ')
	if [ -z "${version}" ]; then
	    echo "Error: Cannot detect fpga version."
	    return 1
	fi

        if [ "${isS2Me}" == "1" ]; then
            echo "it's an S2Me"
            let versiontoflash=0x0109
            let minversiontoflash=0x0106
        elif [ "${isMGB}" = "1" ]; then
            echo "it's an MGB"
            let versiontoflash=0x0109
            let minversiontoflash=0x109
        elif [ "${isQuadE1}" = "1" ]; then
            echo "it's an 4S2M"
            let versiontoflash=0x010a
            let minversiontoflash=0x010a
        elif [ "${isAtomBox}" == "1" ]; then
            echo "it's an OMGA"
            let versiontoflash=0x010c
            let minversiontoflash=0x010c
        else
            echo "nothing to flash"
            return 0;
        fi

	#printf "            version is: %04X\n" ${version}
	#printf "   version to flash is: %04X\n" ${versiontoflash}
	#printf "good enough version is: %04X\n" ${minversiontoflash}

	if [ -z "${version}" -o ${version} -lt ${minversiontoflash} ];then
	    flashit=1
	fi
    done

    if [ "${flashit}" == "1" ];then
	read -p "<notify>You need an FPGA update. After update it is absolutely necessary to powercycle your box. [Continue/Abort/Default=Abort]?" answer
	if [ "${answer}" != "Continue" -a "${answer}" != "C" -a "${answer}" != "c" ]; then
	    echo "Aborting"
	    return 1
	fi
    fi

    return 0
}

echo "Check hardware for FPGA update ..."

isS2Me=$(if lspci -n | grep -q "1011:f001"; then echo 1; else echo 0;fi)
isMGB=$(if lspci -n | grep -q "1011:f002"; then echo 1; else echo 0;fi)
isQuadE1=$(if lspci -n | grep -q "1011:f004"; then echo 1; else echo 0;fi)
isAtomBox=$(if lspci -n | grep -q "1011:f005"; then echo 1; else echo 0;fi)
                
if [ "${isS2Me}" != "1"  -a "${isMGB}" != "1" -a "${isQuadE1}" != "1" -a "${isAtomBox=}" != "1" ]; then
    echo "Hardware without FPGA."
else
    flashit
fi


rpmlist=$(find ${BASE} -name '*.rpm')

if test -z "${rpmlist}"; then
    echo "No Packages found."
    exit 1
fi

grep -q "exit \$RETVAL" /etc/init.d/omgWebInterface
if [ "$?" == 0 ]; then
    echo "repairing defective webstart script"
    sed -i 's/exit \$RETVAL/exit 0/' /etc/init.d/omgWebInterface
fi


yum -y --disablerepo=* remove omg omg-modules || exit 1
yum -y --disablerepo=* install ${rpmlist} || exit 1

#switching to new web ui
sernumstring=$(/usr/bin/i2cinfo -l | grep  -e "OPL\|OPEXT" | head -n 1 | cut -d "," -f 5)
if [ -z $sernumstring ]; then
	let sernum=0
else
	let sernum=$sernumstring
fi
if (( ${sernum} >= 500 )) ; then
	echo "staying on old web ui"
	/usr/bin/omgsetwebui old
else
	echo "switching to new webui"
	/usr/bin/omgsetwebui new
fi

exit 0
