debug = require("debug")("omg-broker:pubsub")
zmq = require("zmq")
child_process = require 'child_process'
fs = require 'fs'

try
  serverPrivateKey = fs.readFileSync "/data/curvekeys/serverPrivate"
catch e
  child_process.execSync "omgcurvekeygen"
  serverPrivateKey = fs.readFileSync("/data/curvekeys/serverPrivate")
serverPrivateKey = serverPrivateKey.toString()

configloader = require 'omg-configloader'
config = configloader.zmq
{standardAddress, standardTransport, pubsubbroker} = config

transport = pubsubbroker.transport or standardTransport
address = pubsubbroker.address or standardAddress

pubPort = "#{transport}://#{address}:#{pubsubbroker.pub}"
subPort = "#{transport}://#{address}:#{pubsubbroker.sub}"

hwm = 1000
verbose = 0

# The xsub listener is where pubs connect to
subSock = zmq.socket("xsub")
subSock.identity = "subscriber" + process.pid
subSock.curve_server = 1
subSock.curve_secretkey = serverPrivateKey
subSock.bindSync subPort

# The xpub listener is where subs connect to
pubSock = zmq.socket("xpub")
pubSock.identity = "publisher" + process.pid
pubSock.setsockopt zmq.ZMQ_SNDHWM, hwm
# By default xpub only signals new subscriptions
# Settings it to verbose = 1 , will signal on every new subscribe
pubSock.setsockopt zmq.ZMQ_XPUB_VERBOSE, verbose
pubSock.curve_server = 1
pubSock.curve_secretkey = serverPrivateKey
pubSock.bindSync pubPort

# When we receive data on subSock , it means someone is publishing
subSock.on "message", (data) ->
  # We just relay it to the pubSock, so subscribers can receive it
  debug "forwarding data to subscribers"
  args = Array.apply null, arguments
  pubSock.send args

# When Pubsock receives a message , it's subscribe requests
pubSock.on "message", (data) ->
  # The data is a slow Buffer
  # The first byte is the subscribe (1) /unsubscribe flag (0)
  type = (if data[0] is 0 then "unsubscribe" else "subscribe")

  # The channel name is the rest of the buffer
  channel = data.slice(1).toString()
  debug "forwarding #{type}: #{channel}"

  # We send it to subSock, so it knows to what channels to listen to
  subSock.send data
  return

